/***************************************************************************
                          calltreedlg.h  -  description
                             -------------------
    begin                : Fri Jul 18 2003
    copyright            : (C) 2003 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CALLTREEDLG_H
#define CALLTREEDLG_H

#include <qwidget.h>
#include <qlistview.h>
#include <calltreelayout.h>
#include "frontend.h"

class CscopeFrontend;

/**
 * Displays a hierarchical tree of calling functions.
 * Each item in the tree is composed of the function name, the path of the
 * source file in which it is defined, the line number and the full text of
 * that line.
 * @author Elad Lahav
 */

class CallTreeDlg : public CallTreeLayout, public FrontendDataTarget
{
   Q_OBJECT
   
public: 
	CallTreeDlg(CscopeFrontend* pCscope, const QString& sFunc,
		QWidget* pParent = 0, const char* szName = 0);
	~CallTreeDlg();

	virtual void dataReady(FrontendToken*);
	virtual void finished(int);

signals:
	/**
	 * Emitted when the user selects an item in the tree view.
	 * @param	sPath	The full path of the file associated with the item
	 * @param	nLine	The line number associated with the item
	 */
	void lineRequested(const QString& sPath, uint nLine);
	
private:
	/** A Cscope process to use for running queries. */
	CscopeFrontend* m_pCscope;
	
	/** The tree item last expended (i.e., for which callers were
		requested.) */
	QListViewItem* m_pActiveItem;

	void queryCallers(const QString&);

private slots:
	void slotExpandFunc(QListViewItem*);
	void slotRequestLine(QListViewItem*);
};

#endif
