/***************************************************************************
                          positionhistory.h
                             -------------------
    begin                : Sat April 3 2004
    copyright            : (C) 2004 by Elad Lahav
    email                : elad_lahav@users.sf.net
 ***************************************************************************/
 
#ifndef POSITIONHISTORY_H
#define POSITIONHISTORY_H

#include <qstring.h>
#include <qvaluelist.h>
#include <qlistview.h>

/**
 * Abstract base class for listing position history entries.
 * @author Elad Lahav
 */
class PositionListTarget
{
public:
	/**
	 * Called when a new history item needs to be added to the target list.
	 * @param	sFilePath	The full path of the source file
	 * @param	nLine		The line number in this source file
	 * @param	nIndex		The index of the item in the list
	 * @param	bCurrent	true if this is the currently selected item
	 */
	virtual void addItem(const QString& sFilePath, uint nLine, int nIndex, 
		bool bCurrent) = 0;
};

/**
 * Keeps track of cursor positions set by Cscope or Ctags queries.
 * @author Elad Lahav
 */
class PositionHistory
{
public:
    PositionHistory();
    ~PositionHistory();
	
	void accept(bool);
	void add(const QString&, uint);
	bool current(QString&, uint&);
	bool next(QString&, uint&);
	bool prev(QString&, uint&);
	void moveTo(int);
	void clear();
	bool fillList(PositionListTarget*);
	
private:
	
	/** Must be set to true for new entries to be added to the history. */
	bool m_bAccept;
	
	/**
	 * Represents a record in the position history list.
	 * A record is composed of a source file's path and a line number.
	 * This class is required to maintain a QValueList collection.
	 * @author	Elad Lahav
	 */
	class FilePosition
	{
	public:
		/**
		 * Default constructor.
		 */
		FilePosition() {}
		
		/**
		 * Class constructor.
		 * @param	sFilePath	The full path of the file field in the record
		 * @param	nLine		The line number field in the record
		 */
		FilePosition(const QString& sFilePath, uint nLine) : 
			m_sFilePath(sFilePath),
			m_nLine(nLine) {}
		
		/** The full path of the source file. */
		QString m_sFilePath;
			
		/** The line number. */
		uint m_nLine;
	};

	/** A list of FilePosition items which serves as the history stack. */
	QValueList<FilePosition> m_list;
	
	/** Points to the current item in the hostory list. */
	QValueList<FilePosition>::Iterator m_itr;
};

extern PositionHistory& PosHistory();

#endif
